==> dtse-1.2-src.tar.bz2: dtse-1.2/Page.cpp <==
/* ============================================================================
 * Douglas Thrift's Search Engine License
 *
 * Copyright (C) 2002-2004, Douglas Thrift. All Rights Reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Douglas Thrift
 *       (http://computers.douglasthrift.net/searchengine/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Douglas Thrift" and "Douglas Thrift's Search Engine" must not
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.  For written permission, please visit
 *    http://www.douglasthrift.net/contact.cgi for contact information.
 *
 * 5. Products derived from this software may not be called "Douglas Thrift's
 *    Search Engine", nor may "Douglas Thrift's Search Engine" appear in their
 *    name, without prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ============================================================================
 */
// Douglas Thrift's Search Engine Page
//
// Douglas Thrift
//
// $Id: Page.cpp,v 1.7 2004/01/01 23:00:34 douglas Exp $

#include "Page.h"

Page::Page(string& url) : URL(url)
{
	size = 0;
}

void Page::setSize(unsigned size)
{
	this->size = size;
}

void Page::setTitle(string& title)
{
	this->title = title;
}

void Page::setDescription(string& description)
{
	this->description = description;
}

void Page::setText(string& text)
{
	this->text = text;
}

void Page::setHeadings(vector<string>& headings)
{
	this->headings = headings;
}

istream& operator>>(istream& is, Page& data)
{
	string line;
#ifdef _OpenSSL_
	bool set = false;
#endif
	
	getline(is, line);
	
	if (line == "	<page>")
	{
		do
		{
			getline(is, line);

			if (line.find("		<address>") == 0)
			{
				unsigned start = line.find("<address>") + 9;
				unsigned finish = line.find("</address>", start);

				string address = line.substr(start, finish - start);

				entities(address, "&lt;", '<');
				entities(address, "&gt;", '>');
				entities(address, "&amp;", '&');

				data.setAddress(address);
			}
			else if (line.find("		<port>") == 0)
			{
				unsigned start = line.find("<port>") + 6;
				unsigned finish = line.find("</port>", start);

				istringstream buffer(line.substr(start, finish - start));
				unsigned port;

				buffer >> port;

				data.setPort(port);

#ifdef _OpenSSL_
				set = true;
#endif
			}
#ifdef _OpenSSL_
			else if (line.find("		<tls>") == 0)
			{
				unsigned start = line.find("<tls>") + 5;
				unsigned finish = line.find("</tls>", start);

				istringstream buffer(line.substr(start, finish - start));
				bool tls;

				buffer.setf(ios_base::boolalpha);
				buffer >> tls;

				data.setTls(tls);

				if (!set && tls) data.setPort(443);
			}
#endif
			else if (line.find("		<path>") == 0)
			{
				unsigned start = line.find("<path>") + 6;
				unsigned finish = line.find("</path>", start);

				string path = line.substr(start, finish - start);

				entities(path, "&lt;", '<');
				entities(path, "&gt;", '>');
				entities(path, "&amp;", '&');

				data.setPath(path);
			}
			else if (line.find("		<size>") == 0)
			{
				unsigned start = line.find("<size>") + 6;
				unsigned finish = line.find("</size>", start);

				istringstream buffer(line.substr(start, finish - start));
				unsigned size;

				buffer >> size;

				data.setSize(size);
			}
			else if (line.find("		<title>") == 0)
			{
				unsigned start = line.find("<title>") + 7;
				unsigned finish = line.find("</title>", start);

				string title = line.substr(start, finish - start);

				while (finish == string::npos)
				{
					getline(is, line);
					finish = line.find("</title>");
					title += '\n' + line.substr(0, finish - 0);
				}

				entities(title, "&lt;", '<');
				entities(title, "&gt;", '>');
				entities(title, "&amp;", '&');

				data.setTitle(title);
			}
			else if (line.find("		<description>") == 0)
			{
				unsigned start = line.find("<description>") + 13;
				unsigned finish = line.find("</description>", start);

				string description = line.substr(start, finish - start);

				entities(description, "&lt;", '<');
				entities(description, "&gt;", '>');
				entities(description, "&amp;", '&');

				data.setDescription(description);
			}
			else if (line.find("		<text>") == 0)
			{
				unsigned start = line.find("<text>") + 6;
				unsigned finish = line.find("</text>", start);

				string text = line.substr(start, finish - start);

				while (finish == string::npos)
				{
					getline(is, line);
					finish = line.find("</text>");
					text += '\n' + line.substr(0, finish - 0);
				}

				entities(text, "&lt;", '<');
				entities(text, "&gt;", '>');
				entities(text, "&amp;", '&');

				data.setText(text);
			}
			else if (line.find("		<heading>") == 0)
			{
				unsigned start = line.find("<heading>") + 9;
				unsigned finish = line.find("</heading>", start);

				string heading = line.substr(start, finish - start);

				while (finish == string::npos)
				{
					getline(is, line);
					finish = line.find("</heading>");
					heading += line.substr(0, finish - 0);
				}

				entities(heading, "&lt;", '<');
				entities(heading, "&gt;", '>');
				entities(heading, "&amp;", '&');

				data.headings.push_back(heading);
			}
		}
		while (line != "	</page>");
	}

	return is;
}

ostream& operator<<(ostream& os, Page& data)
{
	string address = data.getAddress();

	entities(address, '&', "&amp;");
	entities(address, '<', "&lt;");
	entities(address, '>', "&gt;");

	os << "	<page>\n" << "		<address>" << address << "</address>\n";

#ifndef _OpenSSL_
	if (data.getPort() != 80)
#else
	if ((data.getPort() != 80 && !data.getTls()) || (data.getPort() != 443 &&
		data.getTls()))
#endif
	{
		os << "		<port>" << data.getPort() << "</port>\n";
	}

#ifdef _OpenSSL_
	os.setf(ios::boolalpha);
	os << "		<tls>" << data.getTls() << "</tls>\n";
#endif

	string path = data.getPath();

	entities(path, '&', "&amp;");
	entities(path, '<', "&lt;");
	entities(path, '>', "&gt;");

	os << "		<path>" << path << "</path>\n";

	os << "		<size>" << data.getSize() << "</size>\n";

	if(data.getTitle() != "")
	{
		string title = data.getTitle();

		entities(title, '&', "&amp;");
		entities(title, '<', "&lt;");
		entities(title, '>', "&gt;");

		os << "		<title>" << title << "</title>\n";
	}

	if(data.getDescription() != "")
	{
		string description = data.getDescription();

		entities(description, '&', "&amp;");
		entities(description, '<', "&lt;");
		entities(description, '>', "&gt;");

		os << "		<description>" << description << "</description>\n";
	}

	string text = data.getText();

	entities(text, '&', "&amp;");
	entities(text, '<', "&lt;");
	entities(text, '>', "&gt;");

	os << "		<text>" << text << "</text>\n";

	for (int index = 0; index < data.getHeadings().size(); index++)
	{
		string heading = data.getHeadings()[index];

		entities(heading, '&', "&amp;");
		entities(heading, '<', "&lt;");
		entities(heading, '>', "&gt;");

		os << "		<heading>" << heading << "</heading>\n";
	}

	os << "	</page>";

	return os;
}

==> dtse-1.3-src.tar.bz2: dtse-1.3/Page.cpp <==
/* ============================================================================
 * Douglas Thrift's Search Engine License
 *
 * Copyright (C) 2002-2004, 2008, Douglas Thrift. All Rights Reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include the following acknowledgment:
 *
 *       "This product includes software developed by Douglas Thrift
 *       (http://computers.douglasthrift.net/searchengine/)."
 *
 *    Alternately, this acknowledgment may appear in the software itself, if
 *    and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Douglas Thrift" and "Douglas Thrift's Search Engine" must not
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.  For written permission, please visit
 *    http://www.douglasthrift.net/contact.cgi for contact information.
 *
 * 5. Products derived from this software may not be called "Douglas Thrift's
 *    Search Engine", nor may "Douglas Thrift's Search Engine" appear in their
 *    name, without prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ============================================================================
 */
// Douglas Thrift's Search Engine Page
//
// Douglas Thrift
//
// $Id: Page.cpp 372 2008-08-23 11:00:12Z douglas $

#include "Page.hpp"

istream& operator>>(istream& is, Page& data)
{
	string line;
#ifdef _OpenSSL_
	bool set = false;
#endif
	
	getline(is, line);
	
	if (line == "	<page>")
	{
		do
		{
			getline(is, line);

			if (line.find("		<address>") == 0)
			{
				size_t start = line.find("<address>") + 9;
				size_t finish = line.find("</address>", start);

				string address = line.substr(start, finish - start);

				entities(address, "&lt;", '<');
				entities(address, "&gt;", '>');
				entities(address, "&amp;", '&');

				data.setAddress(address);
			}
			else if (line.find("		<port>") == 0)
			{
				size_t start = line.find("<port>") + 6;
				size_t finish = line.find("</port>", start);

				istringstream buffer(line.substr(start, finish - start));
				size_t port;

				buffer >> port;

				data.setPort(port);

#ifdef _OpenSSL_
				set = true;
#endif
			}
#ifdef _OpenSSL_
			else if (line.find("		<tls>") == 0)
			{
				size_t start = line.find("<tls>") + 5;
				size_t finish = line.find("</tls>", start);

				istringstream buffer(line.substr(start, finish - start));
				bool tls;

				buffer.setf(ios_base::boolalpha);
				buffer >> tls;

				data.setTls(tls);

				if (!set && tls) data.setPort(443);
			}
#endif
			else if (line.find("		<path>") == 0)
			{
				size_t start = line.find("<path>") + 6;
				size_t finish = line.find("</path>", start);

				string path = line.substr(start, finish - start);

				entities(path, "&lt;", '<');
				entities(path, "&gt;", '>');
				entities(path, "&amp;", '&');

				data.setPath(path);
			}
			else if (line.find("		<size>") == 0)
			{
				size_t start = line.find("<size>") + 6;
				size_t finish = line.find("</size>", start);

				istringstream buffer(line.substr(start, finish - start));
				size_t size;

				buffer >> size;

				data.setSize(size);
			}
			else if (line.find("		<title>") == 0)
			{
				size_t start = line.find("<title>") + 7;
				size_t finish = line.find("</title>", start);

				string title = line.substr(start, finish - start);

				while (finish == string::npos)
				{
					getline(is, line);
					finish = line.find("</title>");
					title += '\n' + line.substr(0, finish - 0);
				}

				entities(title, "&lt;", '<');
				entities(title, "&gt;", '>');
				entities(title, "&amp;", '&');

				data.setTitle(title);
			}
			else if (line.find("		<description>") == 0)
			{
				size_t start = line.find("<description>") + 13;
				size_t finish = line.find("</description>", start);

				string description = line.substr(start, finish - start);

				entities(description, "&lt;", '<');
				entities(description, "&gt;", '>');
				entities(description, "&amp;", '&');

				data.setDescription(description);
			}
			else if (line.find("		<text>") == 0)
			{
				size_t start = line.find("<text>") + 6;
				size_t finish = line.find("</text>", start);

				string text = line.substr(start, finish - start);

				while (finish == string::npos)
				{
					getline(is, line);
					finish = line.find("</text>");
					text += '\n' + line.substr(0, finish - 0);
				}

				entities(text, "&lt;", '<');
				entities(text, "&gt;", '>');
				entities(text, "&amp;", '&');

				data.setText(text);
			}
			else if (line.find("		<heading>") == 0)
			{
				size_t start = line.find("<heading>") + 9;
				size_t finish = line.find("</heading>", start);

				string heading = line.substr(start, finish - start);

				while (finish == string::npos)
				{
					getline(is, line);
					finish = line.find("</heading>");
					heading += line.substr(0, finish - 0);
				}

				entities(heading, "&lt;", '<');
				entities(heading, "&gt;", '>');
				entities(heading, "&amp;", '&');

				data.headings.push_back(heading);
			}
		}
		while (line != "	</page>");
	}

	return is;
}

ostream& operator<<(ostream& os, Page& data)
{
	string address(data.getAddress());

	entities(address, '&', "&amp;");
	entities(address, '<', "&lt;");
	entities(address, '>', "&gt;");

	os << "\t<page>\n" << "\t\t<address>" << address << "</address>\n";

#ifndef _OpenSSL_
	if (data.getPort() != 80)
#else
	if ((data.getPort() != 80 && !data.getTls()) || (data.getPort() != 443 &&
		data.getTls()))
#endif
	{
		os << "\t\t<port>" << data.getPort() << "</port>\n";
	}

#ifdef _OpenSSL_
	os.setf(ios::boolalpha);

	os << "\t\t<tls>" << data.getTls() << "</tls>\n";
#endif

	string path(data.getPath());

	entities(path, '&', "&amp;");
	entities(path, '<', "&lt;");
	entities(path, '>', "&gt;");

	os << "\t\t<path>" << path << "</path>\n\t\t<size>" << data.getSize()
		<< "</size>\n";

	if(!data.getTitle().empty())
	{
		string title(data.getTitle());

		entities(title, '&', "&amp;");
		entities(title, '<', "&lt;");
		entities(title, '>', "&gt;");

		os << "\t\t<title>" << title << "</title>\n";
	}

	if(!data.getDescription().empty())
	{
		string description(data.getDescription());

		entities(description, '&', "&amp;");
		entities(description, '<', "&lt;");
		entities(description, '>', "&gt;");

		os << "\t\t<description>" << description << "</description>\n";
	}

	string text(data.getText());

	entities(text, '&', "&amp;");
	entities(text, '<', "&lt;");
	entities(text, '>', "&gt;");

	os << "\t\t<text>" << text << "</text>\n";

	for (size_t index(0); index < data.getHeadings().size(); index++)
	{
		string heading(data.getHeadings()[index]);

		entities(heading, '&', "&amp;");
		entities(heading, '<', "&lt;");
		entities(heading, '>', "&gt;");

		os << "\t\t<heading>" << heading << "</heading>\n";
	}

	os << "\t</page>";

	return os;
}
